@@ -1,10 +1,26 @@
 /* getopt.c
 
+   Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+
    -- Revisions -----
-     Logfile:  %F%
-    Revision:  %I%
-        Date:  %G%
-     Modtime:  %U%
+     Logfile:  SCCS/s.getopt.c
+    Revision:  1.2
+        Date:  28 Jul 1997
+     Modtime:  03:16:25
       Author:  @a
    -- SCCS  ---------
 
@@ -28,6 +44,8 @@
 */
 
 #include "getopt.h"
+#include "matutil.h"
+#include "lexerr.h"
 
 /* For communication from `_getopt' to the caller.
    When `_getopt' finds an option that takes an argument,
@@ -153,7 +171,18 @@ static int last_nonopt;
 static void exchange (char **argv)
 {
   int nonopts_size = (last_nonopt - first_nonopt) * sizeof (char *);
-  char **temp = (char **) malloc (nonopts_size);
+  char **temp;
+
+  if (nonopts_size == 0) {
+    printf ("Error: zero length array allocation in exchange - Exiting\n");
+    exit (0);
+  }
+
+  temp = (char **) malloc (nonopts_size);
+
+  if (temp == NULL)
+    ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "exchange", NULL);
+
 
   /* Interchange the two blocks of data in ARGV.  */
 
