@@ -29,9 +29,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.lexfn.c
-    Revision:  1.15
-        Date:  02 Jul 1997
-     Modtime:  08:36:59
+    Revision:  1.17
+        Date:  14 Nov 1997
+     Modtime:  06:38:57
       Author:  @a
    -- SCCS  ---------
 
@@ -309,7 +309,7 @@ BOOL GetNDoses (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
   pifn->nDoses = atoi(szLex);
 
   if ((bErr = (pifn->nDoses <= 0))) {
-    ReportError (pibIn, RE_LEXEXPECTED, "positive-integer", szLex);
+    ReportError (pibIn, RE_LEXEXPECTED | RE_FATAL, "positive-integer", szLex);
     goto Exit_GetNDoses;
   }
 
@@ -341,13 +341,13 @@ BOOL GetNDoses (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
 
     if (!(bOK = GetOptPunct (pibIn, rgszLex[i], ','))) {
       *(rgszLex[i] + 1)  = ',';
-      ReportError(pibIn, RE_EXPECTED, rgszLex[i], NULL);
+      ReportError(pibIn, RE_EXPECTED | RE_FATAL, rgszLex[i], NULL);
       break; /* Error: Stop getting args */
     }
 
     NextLex (pibIn, rgszLex[i], &iType);
     if (!(bOK &= (iType & rgiTypes[i]) > 0))
-      ReportError(pibIn, RE_LEXEXPECTED, vrgszLexTypes[rgiTypes[i]],
+      ReportError(pibIn, RE_LEXEXPECTED | RE_FATAL, vrgszLexTypes[rgiTypes[i]],
                   rgszLex[i]);
   } /* for */
 
@@ -371,7 +371,7 @@ BOOL GetNDoses (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
   pifn->rgT0s[i] = DBL_MAX;
   pifn->rghT0s[i] = 0;
 
-  if (!bOK) ReportError (pibIn, RE_EXPECTED, "input-spec", NULL);
+  if (!bOK) ReportError (pibIn, RE_EXPECTED | RE_FATAL, "input-spec", NULL);
 
 Exit_GetNDoses:
 
@@ -409,7 +409,7 @@ BOOL GetSpikes (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
   pifn->nDoses = atoi(szLex);
 
   if ((bErr = (pifn->nDoses <= 0))) {
-    ReportError (pibIn, RE_LEXEXPECTED, "positive-integer", szLex);
+    ReportError (pibIn, RE_LEXEXPECTED | RE_FATAL, "positive-integer", szLex);
     goto Exit_GetSpikes;
   }
 
@@ -447,7 +447,7 @@ BOOL GetSpikes (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
 
     NextLex (pibIn, rgszLex[i], &iType);
     if (!(bOK &= (iType & rgiTypes[i]) > 0))
-      ReportError(pibIn, RE_LEXEXPECTED, vrgszLexTypes[rgiTypes[i]],
+      ReportError(pibIn, RE_LEXEXPECTED | RE_FATAL, vrgszLexTypes[rgiTypes[i]],
                   rgszLex[i]);
   } /* for */
 
@@ -466,7 +466,7 @@ BOOL GetSpikes (PINPUTBUF pibIn, PSTR szLex, PIFN pifn)
   for (i = 0; i < pifn->nDoses; i++)
     bOK &= DefDepParm (rgszLex[i+pifn->nDoses], pifn->rgT0s+i, pifn->rghT0s+i);
 
-  if (!bOK) ReportError (pibIn, RE_EXPECTED, "input-spec", NULL);
+  if (!bOK) ReportError (pibIn, RE_EXPECTED | RE_FATAL, "input-spec", NULL);
 
 #ifdef ndef 
   /* obsolete code FB 11/6/97 */
