@@ -29,9 +29,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.lexfn.h
-    Revision:  1.10
-        Date:  02 Jul 1997
-     Modtime:  08:36:59
+    Revision:  1.11
+        Date:  14 Nov 1997
+     Modtime:  06:39:13
       Author:  @a
    -- SCCS  ---------
 
