@@ -29,9 +29,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.list.h
-    Revision:  1.8
-        Date:  02 Jul 1997
-     Modtime:  08:37:01
+    Revision:  1.9
+        Date:  14 Nov 1997
+     Modtime:  06:39:13
       Author:  @a
    -- SCCS  ---------
 
