@@ -27,9 +27,9 @@ lsodes2.c
 
    -- Revisions -----
      Logfile:  SCCS/s.lsodes2.c
-    Revision:  1.16
-        Date:  02 Jul 1997
-     Modtime:  08:37:04
+    Revision:  1.17
+        Date:  14 Nov 1997
+     Modtime:  06:39:00
       Author:  @a
    -- SCCS  ---------
 
