@@ -1,20 +1,44 @@
 /* mac.c
 
-   written by Frederic Bois
-   4 September 195
+   Copyright (C) 1989, 1990, 1991, 1992 Free Software Foundation, Inc.
 
-   Copyright (c) 1993.  Don Maszle, Frederic Bois.  All rights reserved.
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License
+   as published by the Free Software Foundation; either version 2
+   of the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 
    -- Revisions -----
      Logfile:  SCCS/s.mac.c
-    Revision:  1.6
-        Date:  02 Jul 1997
-     Modtime:  08:37:05
+    Revision:  1.8
+        Date:  14 Nov 1997
+     Modtime:  06:39:01
       Author:  @a
    -- SCCS  ---------
 
+   FB 21/9/97: this file contains code that is accessed and used only if
+   the symbol _MACOSLEVEL2_ is defined. Level 2 for Macintosh gives a 
+   windows interface and some conformity to the MacOS interface guidelines.
+   The behavior of this file maybe compiler-dependent (I use Symantec's 
+   Think C).
+
+   Level 1 Macintosh implementation is obtained by defining the symbol 
+   _MACOSLEVEL1_ and gives a standard console which should be provided by 
+   the compiler.
 */
 
+#ifdef _MACOSLEVEL2_
+
+#include <PPC Macheaders>
+
 #include <stdlib.h>
 #include <string.h>
 #include <console.h>
@@ -46,6 +70,7 @@ static Point SFGwhere = { 90, 82 };
 static Point SFPwhere = { 106, 104 };
 static SFReply reply;
 
+PANALYSIS panal;
 
 /* -----------------------------------------------------------------------------
  *  AdjustMenus()
@@ -101,14 +126,16 @@ void FileError(Str255 s, Str255 f)
  *    repeatedly (it  handles only one event).
  *
  */
-void HandleEvent(void)
+void HandleEvent (void)
 
 {
-  int ok;
-  EventRecord theEvent;
+  register int ok;
+  static EventRecord theEvent;
 
+  /* disabled
   HiliteMenu(0);
-  SystemTask ();    /* Handle desk accessories */
+  SystemTask ();    Handle desk accessories
+  */
   
   ok = GetNextEvent (everyEvent, &theEvent);
   if (ok)
@@ -126,15 +153,15 @@ void HandleEvent(void)
       }
       break;
       
-    case updateEvt: /*
+ /* case updateEvt: 
       BeginUpdate(theWindow);
       DrawBullseye(((WindowPeek) theWindow)->hilited);
-      EndUpdate(theWindow); */
+      EndUpdate(theWindow); 
       break;
         
-    case activateEvt: /*
-      InvalRect(&theWindow->portRect); */
-      break;
+    case activateEvt: 
+      InvalRect(&theWindow->portRect);
+      break; */
 
   } /* switch */
 
@@ -177,13 +204,14 @@ void HandleMenu (long mSelect)
       case runItem:
         Enable(fileMenu, runItem,  0);
         Enable(fileMenu, stopItem, 1);
+        Enable(fileMenu, quitItem, 1);
         Main_for_mac();
         Enable(fileMenu, stopItem, 0);
         break;
                   
-      case stopItem: ;
-              
+      case stopItem:           
       case quitItem:
+        FreeLevels(panal);
         ExitToShell();
         break;
       }
@@ -272,10 +300,11 @@ void Main_for_mac ()
 {
   PSTR      szFileIn, szFileOut;
   INPUTBUF  ibIn;
-  PANALYSIS panal = (PANALYSIS) malloc (sizeof(ANALYSIS));
   Str255    filename1, filename2;
   int       vRef;
 
+  panal = (PANALYSIS) malloc (sizeof(ANALYSIS));
+
   console_options.nrows = 10;
   console_options.ncols = 60;
   console_options.pause_atexit = 0;
@@ -286,7 +315,10 @@ void Main_for_mac ()
   if (OpenSimFile(filename1, &vRef)) {
     OpenOutFile(filename2, &vRef);
   }
-  else FileError("\pError opening ", filename1);
+  else {
+    FileError("\pError opening ", filename1);
+    return;
+  }
 
   /* cleanup[ the stupid strings */
   filename1[(size_t) filename1[0] + 1] = 0;
@@ -311,10 +343,10 @@ void Main_for_mac ()
   ibIn.pInfo = (PVOID) panal; /* Attach analysis specification to input */
 
   if (ReadAnalysis (&ibIn)) {
-    SetOptions (panal);
     PrepAnalysis (panal);
     DoAnalysis (panal);
   }
+  FreeLevels(panal);
 
 } /* Main_for_mac */
 
@@ -421,5 +453,6 @@ void ShowSplash ()
 
 } /* ShowSplash */
 
+#endif /* _MACOSLEVEL2_ */
 
 /* End */
