@@ -29,25 +29,12 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.mh.c
-    Revision:  1.30
-        Date:  30 Jul 1997
-     Modtime:  06:53:16
+    Revision:  1.34
+        Date:  14 Nov 1997
+     Modtime:  06:39:03
       Author:  @a
    -- SCCS  ---------
 
-   Modified on 18/03/97 by FB: in LnDensity, improved error message for 
-   exponential density.
-
-   Modified on 12/05/97 by FB: in SampleThetas, improved the kernel adjustment.
-  
-   Modified on 21/05/97 by FB: in CheckPrintStatements check that a likelihood
-   is defined (via panal->nModelVars).
-
-   Modified on 22/06/97 by FB: in DoMarkov used pgd->nInitIter to force
-   printing of the times, data, and predictions for the last line of the 
-   restart file
-
-   Modified on 02/07/97 by FB: upgraded SetKernel and SetMCVars.
 */
 
 #include <stdio.h>
@@ -71,7 +58,7 @@ void CalculateTotals (PLEVEL plevel, char **args)
   PANALYSIS panal = (PANALYSIS)args[0];
   double *pdLnPrior = (double*)args[1];
 
-  int n;
+  long n;
 
   for (n = 0; n < plevel->nMCVars; ++n) {
     *pdLnPrior += LnDensity(plevel->rgpMCVars[n], panal);
@@ -90,7 +77,7 @@ void CalculateTotals (PLEVEL plevel, char **args)
 
 void CheckForFixed (PLEVEL plevel, char **args)
 {
-  int     n, m;
+  long    n, m;
   PMCVAR  pMCVar;
   PVARMOD pFVar;
 
@@ -133,7 +120,7 @@ void CheckPrintStatements (PLEVEL plevel, char **args)
   PANALYSIS panal = (PANALYSIS)args[0];
   POUTSPEC pos;
   PMCVAR pMCVar;
-  int n, m, VarIndex = 0, SDIndex;
+  long n, m, VarIndex = 0, SDIndex;
 
   if (plevel->pexpt == NULL)
     return;
@@ -199,11 +186,10 @@ void CheckPrintStatements (PLEVEL plevel, char **args)
    For all MC vars in list at given level, add to arrays of all instances of
    next (lower) level
 */
-
 void CloneMCVars (PLEVEL plevel, char **args)
 {
-  int nMCVars = ListLength(plevel->plistMCVars);
-  int n;
+  long nMCVars = ListLength(plevel->plistMCVars);
+  long n;
   PLEVEL pLower;
 
   for (n = 0; n < plevel->iInstances; ++n) {
@@ -212,22 +198,27 @@ void CloneMCVars (PLEVEL plevel, char **args)
     if (nMCVars != 0) {
       if ((pLower->rgpMCVars = (PMCVAR*) malloc (nMCVars * sizeof(PMCVAR))) ==
           NULL)
-      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "CloneMCVars", NULL);
+        ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "CloneMCVars", NULL);
     }
   }
 
   nMCVars = 0;
   ForAllList3 (plevel->plistMCVars, &CloneMCVarsL, plevel, &nMCVars, NULL);
-  
+
 } /* CloneMCVars */
 
 
+/* Function -------------------------------------------------------------------
+   CloneMCVarsL
+
+   Called from ForAllList3
+*/
 void CloneMCVarsL (PVOID pData, PVOID pUser1, PVOID pUser2, PVOID pUser3)
 {
   PMCVAR pMCVar = (PMCVAR) pData;
   PLEVEL plevel = (PLEVEL) pUser1;
-  int *pnMCVars = (int*) pUser2;
-  int n;
+  long *pnMCVars = (long *) pUser2;
+  long n;
   PLEVEL pLower;
   PMCVAR pClone;
 
@@ -271,7 +262,7 @@ void CloseMarkovFiles (PANALYSIS panal)
 void ConvertLists(PLEVEL plevel, char **args)
 {
   PANALYSIS panal = (PANALYSIS)args[0];
-  int n;
+  long n;
   PMCVAR pMCVar;
 
   ListToPMCArray (panal, panal->plistModelVars, &panal->nModelVars, 
@@ -306,16 +297,23 @@ void DoMarkov (PANALYSIS panal)
 {
   PGIBBSDATA pgd = &panal->gd;
   PLEVEL     pLevel0 = panal->pLevels[0];
-  int        nThetas, nUpdateAt;
-  long       iter = 0;
-  long       nIter = pgd->nMaxIter; /* scheduled iterations of the sampler */
-  double     *pdMCVarVals = NULL;   /* read in values of thetas */
+  long       nThetas, nUpdateAt, nTotal;
+  long       i, iter = 0;
+  long       nIter = pgd->nMaxIter;  /* scheduled iterations of the sampler */
+  double     *pdMCVarVals  = NULL;   /* read in values of thetas */
+  double     *pdSum        = NULL;   /* column sums of read thetas */
+  double     **prgdSumProd = NULL;   /* column sums of thetas cross-products */
   double     dLnPrior = 0, dLnData = 0;
 
   InitRandom (panal->dSeed, TRUE);
 
-  printf ("\nDoing analysis - %ld %s iteration%s\n",
-          nIter, "MCMC sampling", (nIter != 1 ? "s " : " "));
+  if (pgd->nInitIter == 1)
+    printf ("\nDoing analysis -"
+            "\nPrinting data and predictions for the last line of the "
+            "restart file\n");
+  else
+    printf ("\nDoing analysis - %ld %s iteration%s\n",
+            nIter, "MCMC sampling", (nIter != 1 ? "s " : " "));
 
   OpenMarkovFiles (panal);
 
@@ -335,7 +333,7 @@ void DoMarkov (PANALYSIS panal)
 
   /* Now that we have the MC vars right, write the output file header 
      unless it's just a run for fit checking */
-  if (nIter != 0) {
+  if (pgd->nInitIter != 1) {
     fprintf (pgd->pfileOut, "iter\t");
     TraverseLevels (pLevel0, WriteHeader, panal, pgd->pfileOut, NULL);
     fprintf (pgd->pfileOut, "LnPrior\tLnData\tLnPosterior\n");
@@ -364,44 +362,61 @@ void DoMarkov (PANALYSIS panal)
      pointing to the parents' dVal */
   TraverseLevels (pLevel0, SetPointers, panal, NULL);
 
-  /* Get the initial values of the MC vars */
+  /* Get the initial values of the MC vars by reading th restart file if
+     one is given */
   if (pgd->szGrestart) { 
 
     /* Read them from the restart file */
     nThetas = 0;
     TraverseLevels (pLevel0, GetNumberOfMCVars, &nThetas, NULL);
 
-    if ((pdMCVarVals = (double *) malloc (nThetas * sizeof (double))) == NULL)
+    if ( !(pdMCVarVals = InitdVector (nThetas)) ||
+         !(pdSum       = InitdVector (nThetas)) ||
+         !(prgdSumProd = InitdMatrix (nThetas, nThetas)))
       ReportRunTimeError (panal, RE_OUTOFMEM | RE_FATAL, "DoMarkov");
 
     /* Read the starting values in the order they are printed and
        close the file when finished */
-    ReadRestart (pgd->pfileRestart, nThetas, pdMCVarVals, &iter);
+    ReadRestart (pgd->pfileRestart, nThetas, pdMCVarVals, pdSum, prgdSumProd,
+                 &iter);
 
     /* Set the dVals of the variables to the values read in */
-    TraverseLevels (pLevel0, SetMCVars, pdMCVarVals, NULL);
+    nThetas = 0;
+    if (!TraverseLevels1 (pLevel0, SetMCVars, pdMCVarVals, &nThetas, NULL)) {
+      printf ("\nError: some read-in parameters are out of bounds - "
+              "Exiting\n\n");
+      exit(0);
+    }
 
     /* If nInitIter is 1 print just the predictions and data and exit */
     if (pgd->nInitIter == 1) {
-      printf ("\nPrinting data and predictions for the last line "
-              "of the restart file\n");
       PrintAllExpts (pLevel0, 0, panal, pgd->pfileOut);
       CloseMarkovFiles (panal);
       return;
     }
 
-    /* Set the jumping kernel's SD */
-    TraverseLevels (pLevel0, SetKernel, 1, pdMCVarVals, NULL);
+    if (pgd->nInitIter == 0) { /* component jumps */
+      /* Set the jumping kernel's SD */
+      TraverseLevels (pLevel0, SetKernel, 1, pdMCVarVals, NULL);
 
-    /* Free the now used array */
-    free (pdMCVarVals);
+      /* Free the now useless array */
+      free (pdMCVarVals);
+    }
+    else { /* vector jumping, initialize prior */
+      TraverseLevels (pLevel0, CalculateTotals, panal, &dLnPrior, NULL);
+    }
 
     /* Initialize the predictions arrays by running all experiments and
        save the likelihood */
-    RunAllExpts (pLevel0, 0, panal, &dLnData);
+    if (!RunAllExpts (pLevel0, 0, panal, &dLnData)) {
+      /* error, cannot compute */
+      printf ("\nError: cannot compute at the starting point - Exiting\n\n");
+      exit(0);
+    }
   }
 
-  else { 
+  else { /* no restart file, init by sampling */
+
     /* If nInitIter is 1 print an error message and exit */
     if (pgd->nInitIter == 1) {
       printf ("\nError: a restart file must be given to print data and"
@@ -416,12 +431,17 @@ void DoMarkov (PANALYSIS panal)
     fprintf (pgd->pfileOut, "0\t");
     TraverseLevels (pLevel0, InitMCVars, panal, pgd->pfileOut, NULL);
 
-    /* Output log-densities */
-    TraverseLevels(pLevel0, CalculateTotals, panal, &dLnPrior, NULL);
+    /* Calculate the total prior */
+    TraverseLevels (pLevel0, CalculateTotals, panal, &dLnPrior, NULL);
     /* Initilalize the predictions arrays by running all experiments and
        save the data likelihood output */
-    RunAllExpts(pLevel0, 0, panal, &dLnData);
+    if (!RunAllExpts (pLevel0, 0, panal, &dLnData)) {
+      /* error, cannot compute */
+      printf ("\nError: cannot compute at the starting point - Exiting\n\n");
+      exit(0);
+    }
 
+    /* Output prior and likelihood */
     fprintf (pgd->pfileOut, "%e\t%e\t%e\n", dLnPrior, dLnData,
              dLnPrior + dLnData);
     fflush (pgd->pfileOut);
@@ -432,6 +452,7 @@ void DoMarkov (PANALYSIS panal)
 
   /* Initializations are finished, let's do the iterations */
   nUpdateAt = UPDATE_AT;
+  nTotal = UPDATE_AT + 1;
 
   while (iter < nIter) {
 
@@ -444,24 +465,45 @@ void DoMarkov (PANALYSIS panal)
         (iter >= pgd->nMaxIter - pgd->nPrintIter))
       fprintf (pgd->pfileOut, "%ld\t", iter + 1);
 
-    TraverseLevels (pLevel0, SampleThetas, panal, pgd, &iter, &nUpdateAt, 
-                    NULL);
+    if (pgd->nInitIter == 0) { /* component jumps */ 
 
-    /* Output log-densities, eventually */
-    if (((iter + 1) % pgd->nPrintFreq == 0) &&
-        (iter >= pgd->nMaxIter - pgd->nPrintIter)) {  
-      dLnPrior = 0.0;
-      TraverseLevels (pLevel0, CalculateTotals, panal, &dLnPrior, NULL);
-      dLnData = 0.0;     
-      TraverseLevels1 (pLevel0, SumAllExpts, &dLnData, NULL);
-      fprintf (pgd->pfileOut, "%e\t%e\t%e\n", dLnPrior, dLnData,
+      TraverseLevels (pLevel0, SampleThetas, panal, pgd, &iter, &nUpdateAt, 
+                      &nTotal, NULL);
+
+      /* Output log-densities, eventually */
+      if (((iter + 1) % pgd->nPrintFreq == 0) &&
+          (iter >= pgd->nMaxIter - pgd->nPrintIter)) {  
+        dLnPrior = 0.0;
+        TraverseLevels (pLevel0, CalculateTotals, panal, &dLnPrior, NULL);
+        dLnData = 0.0;     
+        TraverseLevels1 (pLevel0, SumAllExpts, &dLnData, NULL);
+        fprintf (pgd->pfileOut, "%e\t%e\t%e\n", dLnPrior, dLnData,
                dLnPrior + dLnData);
-      fflush (pgd->pfileOut);
+        fflush (pgd->pfileOut);
+      }
+    }
+    else { /* vector jumps */
+
+      SampleThetaVector (pLevel0, panal, nThetas, pdMCVarVals, pdSum, 
+                         prgdSumProd, iter, nUpdateAt, &dLnPrior, &dLnData);
+
+      /* Output log-densities, eventually */
+      if (((iter + 1) % pgd->nPrintFreq == 0) &&
+          (iter >= pgd->nMaxIter - pgd->nPrintIter)) { 
+        for (i = 0; i < nThetas; i++) 
+          fprintf(pgd->pfileOut, "%5g\t", pdMCVarVals[i]);
+        fprintf (pgd->pfileOut, "%e\t%e\t%e\n", dLnPrior, dLnData,
+               dLnPrior + dLnData);
+        fflush (pgd->pfileOut);
+      }
     }
 
+
     /* Adjust the update time eventually */
-    if ((iter % nUpdateAt == 0) && (iter > 0))
+    if (iter == nUpdateAt) {
       nUpdateAt = nUpdateAt * 1.5;
+      nTotal = nUpdateAt - iter;
+    }
 
     /* Increment the iteration counter */
     ++iter;
@@ -484,7 +526,7 @@ void DoMarkov (PANALYSIS panal)
 void FindMCParents (PLEVEL plevel, char **args)
 {
   PANALYSIS panal = (PANALYSIS)args[0];
-  int       n, m, l, k;
+  long      n, m, l, k;
   PLEVEL    pPrevLev;
   PMCVAR    pMCVar1, pMCVar2;
 
@@ -534,7 +576,7 @@ void FindMCParents (PLEVEL plevel, char **args)
 
 void FindMCDependents (PLEVEL plevel, char **args)
 {
-  int n, m;
+  long n, m;
 
   for (n = 0; n < plevel->nMCVars; ++n)
     for (m = 0; m < 4; ++m)
@@ -555,7 +597,7 @@ void FindMCDependents (PLEVEL plevel, char **args)
 */
 void GetNumberOfMCVars (PLEVEL plevel, char **args) 
 {
-  int *pnThetas = (int*) args[0];
+  long *pnThetas = (long *) args[0];
 
   *pnThetas += plevel->nMCVars;
 
@@ -573,7 +615,7 @@ void InitMCVars(PLEVEL plevel, char **args)
 {
   PANALYSIS panal = (PANALYSIS)args[0];
   FILE      *pOutFile = (FILE*)args[1];
-  int       n;
+  long      n;
 
   for (n = 0; n < plevel->nMCVars; ++n) 
     if ( !(plevel->rgpMCVars[n]->bIsFixed))
@@ -876,7 +918,7 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal)
 
 double LnLike (PMCVAR pMCVar, PANALYSIS panal) 
 {
-  int n;
+  long n;
   double dDensity, dLnLike = 0.0;
 
   for (n = 0; n < pMCVar->nDependents; ++n)
@@ -900,42 +942,50 @@ double LnLikeData (PEXPERIMENT pexpt, PANALYSIS panal) {
 
   POUTSPEC pos;
   PMCVAR pMCVar;
-  int n, m, l;
+  long n, m, l;
   double dLnLike = 0.0;
   double dTmp;
 
   pos = &pexpt->os;
 
+  /* scan all the model states and outputs */
   for (n = 0; n < panal->nModelVars; ++n) {
     pMCVar = panal->rgpModelVars[n];
 
+    /* scan all requested outputs in simulaion file */
     for (m = 0; m < pos->nOutputs; ++m) {
+      /* if model variable and output handles match, process */
       if (pos->phvar[m] == pMCVar->hvar) {
-        for (l = 0; l < pos->pcOutputTimes[m]; ++l) {
-          if (pos->prgdOutputVals[m][l] != MISSING_VALUE) {
+        /* for each requested output time */
+        for (l = 0; l < pos->pcOutputTimes[m]; ++l) 
+          if (pos->prgdDataVals[m][l] != INPUT_MISSING_VALUE) {
+            /* if the data is not coded as missing */
+            if (pos->prgdOutputVals[m][l] != MISSING_VALUE) {
+              /* if the model output is not coded as missing */
 
-            pMCVar->dVal = pos->prgdDataVals[m][l];
-            pMCVar->dParm[0] = pos->prgdOutputVals[m][l];
+              pMCVar->dVal = pos->prgdDataVals[m][l]; /* data value */
+              pMCVar->dParm[0] = pos->prgdOutputVals[m][l]; /* predictions */
 
-            /* This finds the SD to use */
-            if (pMCVar->cVarParm >> 1 & MCVP_VARIABLE) 
-              if (!IsModelVar (pMCVar->hParm[1])) {
-                pMCVar->dParm[1] = GetVarValue (pMCVar->hParm[1]);
-                /* This is not very clean and the dVal of a MCVar structure
-                   should be used instead */
-              }
-              else
-                pMCVar->dParm[1] = pos->prgdOutputVals[pMCVar->iParmIndex][l];
+              /* This finds the SD to use */
+              if (pMCVar->cVarParm >> 1 & MCVP_VARIABLE) 
+                if (!IsModelVar (pMCVar->hParm[1])) {
+                  pMCVar->dParm[1] = GetVarValue (pMCVar->hParm[1]);
+                  /* This is not very clean and the dVal of a MCVar structure
+                     should be used instead */
+                }
+                else
+                  pMCVar->dParm[1] = 
+                    pos->prgdOutputVals[pMCVar->iParmIndex][l];
             
-            dTmp = LnDensity (pMCVar, panal);
-            if (dTmp == (BAD_VAL))
-              return (BAD_VAL);
+              dTmp = LnDensity (pMCVar, panal);
+              if (dTmp == (BAD_VAL))
+                return (BAD_VAL);
+              else
+                dLnLike = dLnLike + dTmp;
+            }
             else
-              dLnLike = dLnLike + dTmp;
+              ReportRunTimeError (panal, RE_BADMODEL | RE_FATAL, "LnLikeData");
           }
-          else
-            ReportRunTimeError (panal, RE_BADMODEL | RE_FATAL, "LnLikeData");
-        }
       }
     }
   }
@@ -983,9 +1033,9 @@ void OpenMarkovFiles (PANALYSIS panal)
    Run all experiments at level below `plevel'
 */
 
-void PrintAllExpts (PLEVEL plevel, int nVar, PANALYSIS panal, PFILE pOutFile)
+void PrintAllExpts (PLEVEL plevel, long nVar, PANALYSIS panal, PFILE pOutFile)
 {
-  int n;
+  long n;
 
   for (n = 0; n < plevel->iInstances; ++n)
     TraverseLevels1 (plevel->pLevels[n], PrintExpt, panal, pOutFile, NULL);
@@ -996,9 +1046,8 @@ void PrintAllExpts (PLEVEL plevel, int nVar, PANALYSIS panal, PFILE pOutFile)
 /* Function -------------------------------------------------------------------
    PrintExpt
 
-   If `plevel' has no experiments, modify the variables using its two
-   lists; if it has experiments, run them. Return 1 on success and 0 if
-   failure
+   Run all experiments and print the level code, experiment number, time, data,
+   and predictions to the output file.
 
    Called from TraverseLevels1
 */
@@ -1007,19 +1056,22 @@ int PrintExpt (PLEVEL plevel, char **args)
 {
   PANALYSIS   panal = (PANALYSIS)args[0];
   PFILE       pOutFile = (PFILE)args[1];
-  int         l, m, n;
+  long        l, m, n;
   PEXPERIMENT pExpt = plevel->pexpt;
   POUTSPEC    pos;
-  static int  printed_head = 0;
+  static long printed_head = 0;
 
   if (!printed_head) {
-    fprintf (pOutFile, "Time\tData\tPrediction\n");
+    fprintf (pOutFile, 
+             "Level\tExperiment\tOutput_Var\tTime\tData\tPrediction\n");
     printed_head = 1;
   }
 
   /* Set level sequence */
   panal->pCurrentLevel[plevel->iDepth] = plevel;
 
+  panal->iInstance[plevel->iDepth] = plevel->iSequence;
+
   if (pExpt != NULL) {
     InitModel ();
 
@@ -1039,7 +1091,12 @@ int PrintExpt (PLEVEL plevel, char **args)
       pos = &pExpt->os;
       for (m = 0; m < pos->nOutputs; ++m) {
         for (l = 0; l < pos->pcOutputTimes[m]; ++l) {
-          fprintf (pOutFile, "%g\t%g\t%g\n", pos->prgdOutputTimes[m][l], 
+          for (n = 1; n < plevel->iDepth; ++n)
+            fprintf (pOutFile, "%d.", panal->iInstance[n]);
+          fprintf (pOutFile, "%d\t", panal->iInstance[plevel->iDepth]);
+          fprintf (pOutFile, "%d\t%s\t%g\t%g\t%g\n", pExpt->iExp,
+                   pos->pszOutputNames[m],
+                   pos->prgdOutputTimes[m][l], 
                    pos->prgdDataVals[m][l], pos->prgdOutputVals[m][l]);
         }
       }
@@ -1062,7 +1119,7 @@ int PrintExpt (PLEVEL plevel, char **args)
 
 void PrintDeps (PLEVEL plevel, char **args)
 {
-  int n, m;
+  long n, m;
   PMCVAR pMCVar;
 
   fprintf (stderr, "Depth %d; Instance %d\n", 
@@ -1076,7 +1133,7 @@ void PrintDeps (PLEVEL plevel, char **args)
 
     for (m = 0; m < 4; ++m)
       if (pMCVar->pMCVParent[m] != NULL)
-        fprintf (stderr, "  Parent %d: %s (%d) [%lx]\n", m,
+        fprintf (stderr, "  Parent %ld: %s (%d) [%lx]\n", m,
                  pMCVar->pMCVParent[m]->pszName,
                  pMCVar->pMCVParent[m]->iDepth, 
                  (unsigned long) pMCVar->pMCVParent[m]);
@@ -1102,12 +1159,19 @@ void PrintDeps (PLEVEL plevel, char **args)
 */
 
 void ReadRestart (FILE *pfileRestart, long nThetas,
-                  double *pdTheta, long *pIter) 
+                  double *pdTheta, double *pdSum, double **prgdSumProd, 
+                  long *pIter) 
 {
-  register int n;
+  register long i, j;
 
   *pIter = -1;
 
+  for (i = 0; i < nThetas; i++) {
+    pdSum[i] = 0.0;
+    for (j = 0; j < nThetas; j++)
+      prgdSumProd[i][j] = 0.0;
+  }
+
   /* skip the first line.  This allows a MC output file to be used
      directly as a restart file. */
   fscanf (pfileRestart,  "%*[^\n]");  getc(pfileRestart);
@@ -1119,17 +1183,27 @@ void ReadRestart (FILE *pfileRestart, long nThetas,
    */
   while (!(feof (pfileRestart) ||
           (fscanf (pfileRestart, "%*s") == EOF))) {
-    for (n = 0; n < nThetas; ++n) {
-      if (fscanf(pfileRestart, "%lg", &(pdTheta[n])) == EOF) {
+    for (i = 0; i < nThetas; i++) {
+      if (fscanf(pfileRestart, "%lg", &(pdTheta[i])) == EOF) {
         printf ("Error: incorrect length for restart file - Exiting\n");
         exit(0);
       }
+      else { /* reading ok */
+        /* update pdSum, the column sum of pdTheta */
+        pdSum[i] = pdSum[i] + pdTheta[i];
+      }
     }
 
     /* Throw away remainder of line. This allows a MC output file to be used
        directly as a restart file. */
     fscanf (pfileRestart,  "%*[^\n]"); getc(pfileRestart);
 
+    /* update prgdSumProd */
+    for (i = 0; i < nThetas; i++)
+      for (j = 0; j < nThetas; j++)
+        prgdSumProd[i][j] = prgdSumProd[i][j] + pdTheta[i] * pdTheta[j];
+
+
     /* increment pIter */
     *pIter = *pIter + 1;
 
@@ -1167,12 +1241,20 @@ int RestoreLikelihoods (PLEVEL plevel, char **args)
    Run all experiments at level below `plevel'
 */
 
-void RunAllExpts (PLEVEL plevel, int nVar, PANALYSIS panal, double *pdLnData)
+int RunAllExpts (PLEVEL plevel, long nVar, PANALYSIS panal, PDOUBLE pdLnData)
 {
-  int n;
+  long n;
 
-  for (n = 0; n < plevel->iInstances; ++n)
-    TraverseLevels1 (plevel->pLevels[n], RunExpt, panal, pdLnData, NULL);
+  for (n = 0; n < plevel->iInstances; ++n) {
+    if (!TraverseLevels1 (plevel->pLevels[n], RunExpt, panal, 
+                          pdLnData, NULL)) {
+      /* error */
+      return (0);
+    }
+  }
+  
+  /* success */
+  return (1);
 
 } /* RunAllExpts */
 
@@ -1189,9 +1271,9 @@ void RunAllExpts (PLEVEL plevel, int nVar, PANALYSIS panal, double *pdLnData)
 
 int RunExpt (PLEVEL plevel, char **args)
 {
-  PANALYSIS   panal = (PANALYSIS)args[0];
-  double      *pdLnData = (double*)args[1];
-  int         n;
+  PANALYSIS   panal = (PANALYSIS) args[0];
+  double      *pdLnData = (double *) args[1];
+  long        n;
   PEXPERIMENT pExpt = plevel->pexpt;
 
   /* Set level sequence */
@@ -1260,17 +1342,19 @@ double SampleTheta (PMCVAR pMCVar) {
 
 void SampleThetas (PLEVEL plevel, char **args)
 {
-  PANALYSIS panal = (PANALYSIS)args[0];
-  PGIBBSDATA pgd = (PGIBBSDATA)args[1];
-  long   *pnIter = (long*)args[2];
-  int    *pnUpdateAt = (int*)args[3];
+  PANALYSIS  panal = (PANALYSIS) args[0];
+  PGIBBSDATA pgd = (PGIBBSDATA) args[1];
+  long *pnIter = (long *) args[2];
+  long *pnUpdateAt = (long *) args[3];
+  long *pnTotal = (long *) args[4];
+  
 
   double dLnPrior, dLnLike, dLnData;
   double dLnPriorNew, dLnLikeNew, dLnDataNew;
   double dTheta, dJumps, dSum, dVar;
   PMCVAR pMCVar;
   static MCVAR tmpMCVar;
-  int i, n;
+  long i, n;
 
   /* For all MC vars at this level */
   for (n = 0; n < plevel->nMCVars; ++n) {
@@ -1323,20 +1407,63 @@ void SampleThetas (PLEVEL plevel, char **args)
   
       /* Adjust the jumping kernel SD, depending on acceptance rates, 
          make sure it does not exceed DBL_MAX or a third of the range */
-      if ((*pnIter % *pnUpdateAt == 0) && (*pnIter > 0)) {
-        if (((dJumps = (double)pMCVar->lJumps / (double)*pnUpdateAt) > 0.3) &&
-            (pMCVar->dKernelSD < ((*(pMCVar->pdParm[3]) / 6) - 
-                                  (*(pMCVar->pdParm[2]) / 6))) &&
-            (pMCVar->dKernelSD < (DBL_MAX / 2)))
-          pMCVar->dKernelSD = pMCVar->dKernelSD * 2;
+      if (*pnIter == *pnUpdateAt) {
+
+        dJumps = (double) pMCVar->lJumps / (double) (*pnTotal);
+
+        /* the kernel should not be infinitely increased
+        if (pMCVar->iType == MCV_UNIFORM || pMCVar->iType == MCV_LOGUNIFORM )
+          dMaxSD = (*(pMCVar->pdParm[1]) / 6.0) - (*(pMCVar->pdParm[0]) / 6.0);
         else 
-          if (dJumps < 0.15)
-            pMCVar->dKernelSD = pMCVar->dKernelSD * 0.5;
+          dMaxSD = (*(pMCVar->pdParm[3]) / 6.0) - (*(pMCVar->pdParm[2]) / 6.0);
+        removed by FB 5/11/1997
+        */
+
+        if (dJumps > 0.3) { 
+          if (dJumps > 0.99999) { 
+            /* pathological case: chances are the parameter has such a 
+               small kernel that it does not change values, this leads
+               to a "jump" each time. Drastic increase of the kernel is
+               attempted */
+            if (pMCVar->dKernelSD < sqrt(DBL_MAX))
+              pMCVar->dKernelSD = pMCVar->dKernelSD * pMCVar->dKernelSD;
+            else
+              pMCVar->dKernelSD = DBL_MAX;
+          }
+          else { /* more normal case */
+            /* the kernel should not be infinitely increased */
+            if (pMCVar->dKernelSD < DBL_MAX / 2)
+              pMCVar->dKernelSD = pMCVar->dKernelSD * 2;
+            else
+              pMCVar->dKernelSD = DBL_MAX;
+          }
+        }
+        else {
+          if (dJumps < 0.00001) { 
+            /* pathological case: chances are the parameter has such a 
+               big kernel that it will never jump. Drastic decrease of 
+               the kernel is attempted, dissymetric from the increase */
+            if (pMCVar->dKernelSD > pow(DBL_MIN, 0.45))
+              pMCVar->dKernelSD = pow(pMCVar->dKernelSD, 0.45);
+            else
+              pMCVar->dKernelSD = DBL_MIN;
+          }
+          else { /* more normal case */
+            /* the kernel should not be infinitely decreased; decrease
+               is otherwise slighly different from the increase to avoid
+               oscillations */
+            if (pMCVar->dKernelSD > DBL_MIN / 0.4)
+              pMCVar->dKernelSD = pMCVar->dKernelSD * 0.4;
+            else
+              pMCVar->dKernelSD = DBL_MIN;
+          }
+        }
   
-        pMCVar->lJumps = 0;
+        pMCVar->lJumps = 0; /* reset the jumps counter */
+
       }
   
-      /* Sample a new value */
+      /* sample a new value */
       pMCVar->dVal = SampleTheta (pMCVar);
   
       /* recompute prior and likelihood */
@@ -1359,9 +1486,8 @@ void SampleThetas (PLEVEL plevel, char **args)
       }      
   
       /* Test the results and act accordingly */
-      if (!Test (pMCVar->bExptIsDep, *pnIter, pgd->nInitIter,
-                 dLnPrior, dLnPriorNew, dLnLike, dLnLikeNew,
-                 dLnData, dLnDataNew)) {
+      if (!Test (pMCVar->bExptIsDep,  dLnPrior, dLnPriorNew, dLnLike, 
+                 dLnLikeNew, dLnData, dLnDataNew)) {
         pMCVar->dVal = dTheta;
   
         if(pMCVar->bExptIsDep)
@@ -1386,6 +1512,152 @@ WriteIt: /* Write the MC var value to output file */
 } /* SampleThetas */
 
 
+/* Function -------------------------------------------------------------------
+   SampleThetaVector
+
+   Sample thetas in block, do Metropolis test, write the sampled values to 
+   the output file
+*/
+
+void SampleThetaVector (PLEVEL pLevel, PANALYSIS panal, long nThetas,
+                        double *pdTheta, double *pdSum, double **prgdSumProd,
+                        long iter, long nUpdateAt, PDOUBLE pdLnPrior, 
+                        PDOUBLE pdLnData)
+{
+  register long i, j;
+  double dTmp, dAccept, dLnPrior_old, dLnData_old;
+
+  static long lAccepted = 0;
+  static long lOldUpdate = 1;
+  static double dJumpSpread;
+  static PDOUBLE pdTheta_old = NULL; /* previous model parameters values */
+  static PDOUBLE *prgdComponent;
+  static PDOUBLE *prgdVariance;
+  static PDOUBLE dNormVar; /* storage for nParms normal deviates */
+
+  if ((pdTheta_old == NULL) || (iter == nUpdateAt)) {
+
+    if (pdTheta_old == NULL) { /* initialize */
+      if ( !(pdTheta_old = InitdVector (nThetas)) ||
+           !(dNormVar    = InitdVector (nThetas)) ||
+           !(prgdVariance  = InitdMatrix (nThetas, nThetas)) ||
+           !(prgdComponent = InitdMatrix (nThetas, nThetas)))
+        ReportRunTimeError (panal, RE_OUTOFMEM | RE_FATAL, 
+                            "SampleThetaVector");
+
+      /* initialize dJumpSpread */
+      dJumpSpread = 2.4 / sqrt(nThetas); /* Gelman's Normal theory result */
+    }
+    else {
+      /* if some vector samplings have been made, check that the current
+         dJumpSpread leads to an acceptation rate of 15 to 30% over the
+         last batch of simulations. Adjust eventually. */
+      dAccept = ((double) lAccepted) / (double) (nUpdateAt - lOldUpdate);
+
+      if ( dAccept > 0.3) dJumpSpread = dJumpSpread * 1.5;
+      else if (dAccept < 0.15) dJumpSpread = dJumpSpread / 1.5;
+
+      printf ("Monitoring: iter\t%ld\t", iter);
+      printf ("success rate\t%g\tspread\t%g\n", dAccept, dJumpSpread);
+      lAccepted = 0; /* reset the counter */
+      lOldUpdate = nUpdateAt;
+    }
+
+    /* other updates: */
+
+    /* generate the covariance matrix */
+    for (i = 0; i < nThetas; i++)
+      for (j = 0; j < nThetas; j++)
+        prgdVariance[i][j] = (prgdSumProd[i][j] - 
+                              pdSum[i] * pdSum[j] / (long) (iter+1)) / 
+                              (long) iter;
+
+    /* do the Cholesky decomposition of the covariance matrix */
+    Cholesky (prgdVariance, prgdComponent, nThetas);
+
+    /* do the Cholesky decomposition of the covariance matrix */
+    if (!Cholesky (prgdVariance, prgdComponent, nThetas)) {
+      /* try to save the computation by zeroing all non-diagonal elements */
+      for (i = 0; i < nThetas; i++)
+        for (j = 0; j < nThetas; j++)
+        	if (i == j)
+            prgdVariance[i][j] = prgdSumProd[i][j] / (double) (iter);
+          else
+            prgdVariance[i][j] = 0.0;
+
+    	/* if is still does not work, exit */
+      if (!Cholesky (prgdVariance, prgdComponent, nThetas)) {
+      	printf ("Error: impossible to compute a jumping kernel - Exiting."
+                "(You should check or change the restart file).\n\n");
+      	exit (0);
+      }   
+    }
+  }
+
+  /* keep the value of all thetas */
+  for (i = 0; i < nThetas; i++)
+    pdTheta_old[i] = pdTheta[i];
+
+  /* keep old prior and old likelihood */
+  dLnPrior_old = *pdLnPrior; 
+  dLnData_old  = *pdLnData;
+
+  /* generate new pdTheta vector in the range of each element */
+  do {
+    for (i = 0; i < nThetas; i++) dNormVar[i] = NormalRandom(0, 1);
+
+    for (i = 0; i < nThetas; i++) {
+      dTmp = 0;
+      for (j = 0; j <= i; j++) /* only the non-zero part of prgdComponent */
+        dTmp = dTmp + dNormVar[j] * prgdComponent[i][j];
+
+      pdTheta[i] = pdTheta_old[i] + dJumpSpread * dTmp;
+    }
+
+    /* Set the dVals of the variables to the values sampled and check that
+       we are within bounds as a while condition */
+    nThetas = 0;
+  } while (!TraverseLevels1 (pLevel, SetMCVars, pdTheta, &nThetas, NULL));
+
+  /* Calculate the new prior */
+  *pdLnPrior = 0.0;
+  TraverseLevels (pLevel, CalculateTotals, panal, pdLnPrior, NULL);
+
+  /* compute the model at the newly drawn point and the likelihood */
+  *pdLnData = 0.0;
+  if (!RunAllExpts (pLevel, 0, panal, pdLnData)) {
+    /* computation failed, don't jump, restore */
+    for (i = 0; i < nThetas; i++)
+      pdTheta[i] = pdTheta_old[i];
+    *pdLnPrior = dLnPrior_old; 
+    *pdLnData  = dLnData_old;
+  }
+  else {
+    /* Test */
+    if (log(Randoms()) > 
+        ((*pdLnPrior) + (*pdLnData) - dLnPrior_old - dLnData_old)) { 
+      /* don't jump, restore */
+      for (i = 0; i < nThetas; i++)
+        pdTheta[i] = pdTheta_old[i];
+      *pdLnPrior = dLnPrior_old; 
+      *pdLnData  = dLnData_old;
+    }
+    else { /* jump */
+      lAccepted++; /* this is used above to adjust the acceptation rate */
+    }
+  }
+
+  /* update arrays */
+  for (i = 0; i < nThetas; i++) {
+    pdSum[i] = pdSum[i] + pdTheta[i];
+    for (j = 0; j < nThetas; j++)
+      prgdSumProd[i][j] = prgdSumProd[i][j] + pdTheta[i] * pdTheta[j];
+  }
+
+
+} /* SampleThetaVector */
+
+
 /* Function -------------------------------------------------------------------
    SaveLikelihoods
 
@@ -1413,7 +1685,7 @@ int SaveLikelihoods (PLEVEL plevel, char **args)
 
 void SetFixedVars (PLEVEL plevel)
 {
-  int n;
+  long n;
   PVARMOD pFVar;
 
   for (n = 0; n < plevel->nFixedVars; ++n) {
@@ -1497,7 +1769,7 @@ void SetKernel (PLEVEL plevel, char **args)
 
 void SetModelVars(PLEVEL plevel)
 {
-  int n;
+  long n;
   PMCVAR  pMCVar;
 
   for (n = 0; n < plevel->nMCVars; ++n) {
@@ -1512,19 +1784,41 @@ void SetModelVars(PLEVEL plevel)
 /* Function -------------------------------------------------------------------
    SetMCVars
 
-   Set initial values of thetas after reading input file -
-   values are assumed to be in proper order
-   Called from TraverseLevels
+   Set initial values of thetas after reading input file or sampling them.
+   Values are assumed to be in proper order.
+   It also checks that the ranges are respected.
+   Called from TraverseLevels.
 */
 
-void SetMCVars (PLEVEL plevel, char **args)
+int SetMCVars (PLEVEL plevel, char **args)
 {
-  double *pdMCVarVals = (double*) args[0];
-  static long nThetas = 0;
+  double *pdMCVarVals = (double *) args[0];
+  long   *nThetas = (long *) args[1];
+  PMCVAR pMCVar;
+  double dVar;
   long   n;
 
-  for (n = 0; n < plevel->nMCVars; ++n)
-    plevel->rgpMCVars[n]->dVal = pdMCVarVals[nThetas++];
+  for (n = 0; n < plevel->nMCVars; ++n) {
+    dVar = pdMCVarVals[(*nThetas)++];
+    pMCVar = plevel->rgpMCVars[n];
+    if ((pMCVar->iType == MCV_UNIFORM) || (pMCVar->iType == MCV_LOGUNIFORM)) {
+      if ((dVar < *(pMCVar->pdParm[0])) || (dVar > *(pMCVar->pdParm[1]))) {
+        /* error */
+        return (0);
+      }
+    }
+    else {
+      if ((dVar < *(pMCVar->pdParm[2])) || (dVar > *(pMCVar->pdParm[3]))) {
+        /* error */
+        return (0);
+      }
+    }
+     
+    pMCVar->dVal = dVar;
+  }
+
+  /* success */
+  return (1);
 
 } /* SetMCVars */
 
@@ -1541,7 +1835,7 @@ void SetMCVars (PLEVEL plevel, char **args)
 
 void SetPointers (PLEVEL plevel, char **args)
 {
-  int n, m;
+  long n, m;
   PMCVAR pMCVar;
 
   for (n = 0; n < plevel->nMCVars; ++n) {
@@ -1587,10 +1881,10 @@ int SumAllExpts (PLEVEL plevel, char **args)
    Test prior, likelihood against random number between 0 and 1
 */
 
-BOOL Test (BOOL bExptIsDep, long iter, long initIter, double dLnPrior,
-           double dLnPriorNew, double dLnLike, double dLnLikeNew,
-           double dLnData, double dLnDataNew) {
-
+BOOL Test (BOOL bExptIsDep, double dLnPrior, double dLnPriorNew,
+           double dLnLike, double dLnLikeNew,
+           double dLnData, double dLnDataNew) 
+{
   double dPjump;
 
   if (dLnPriorNew == BAD_VAL || dLnLikeNew == BAD_VAL)
@@ -1687,7 +1981,7 @@ void WriteHeader (PLEVEL plevel, char **args)
 {
   PANALYSIS panal = (PANALYSIS)args[0];
   FILE *outFile = (FILE*)args[1];
-  int n, m;
+  long n, m;
 
   panal->iInstance[plevel->iDepth] = plevel->iSequence;
 
@@ -1698,7 +1992,7 @@ void WriteHeader (PLEVEL plevel, char **args)
     fprintf (outFile, "%d)\t", panal->iInstance[plevel->iDepth]);
   }
 
-  fflush (outFile);
+  /* fflush (outFile); */
 
 } /* WriteHeader */
 
@@ -1712,7 +2006,7 @@ void WriteHeader (PLEVEL plevel, char **args)
 
 void WriteMCVars (PLEVEL plevel, PANALYSIS panal, PFILE pOutFile)
 {
-  int n;
+  long n;
   PMCVAR pMCVar;
 
   for (n = 0; n < plevel->nMCVars; ++n) {
