@@ -28,9 +28,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.mh.h
-    Revision:  1.10
-        Date:  18 Jul 1997
-     Modtime:  09:16:21
+    Revision:  1.12
+        Date:  14 Nov 1997
+     Modtime:  06:39:17
       Author:  @a
    -- SCCS  ---------
 
@@ -112,33 +112,39 @@ double LnDensity (PMCVAR pMCVar, PANALYSIS panal);
 double LnLike (PMCVAR pMCVar, PANALYSIS panal);
 double LnLikeData (PEXPERIMENT pexpt, PANALYSIS panal);
 void   OpenMarkovFiles (PANALYSIS panal);
-void   PrintAllExpts (PLEVEL plevel, int nVar, PANALYSIS panal,
+void   PrintAllExpts (PLEVEL plevel, long nVar, PANALYSIS panal,
                       PFILE pOutFile);
 void   PrintDeps (PLEVEL plevel, char **args);
 int    PrintExpt (PLEVEL plevel, char **args);
 void   ReadRestart (FILE *pfileRestart, long nThetas,
-                    double *pdTheta, long *pIter);
+                    double *pdTheta, double *pdSum, double **prgdSumProd, 
+                    long *pIter);
 int    RestoreLikelihoods (PLEVEL plevel, char **args);
-void   RunAllExpts(PLEVEL plevel, int nVar, PANALYSIS panal, double *pdLnData);
+int    RunAllExpts(PLEVEL plevel, long nVar, PANALYSIS panal, 
+       PDOUBLE pdLnData);
 int    RunExpt (PLEVEL plevel, char **args);
 double SampleTheta (PMCVAR pMCVar);
 void   SampleThetas (PLEVEL plevel, char **args);
+void   SampleThetaVector (PLEVEL pLevel, PANALYSIS panal, long nThetas,
+                          double *pdTheta, double *pdSum, double **prgdSumProd,
+                          long iter, long nUpdateAt, PDOUBLE pdLnPrior, 
+                          PDOUBLE pdLnData);
 int    SaveLikelihoods (PLEVEL plevel, char **args);
 void   SetFixedVars (PLEVEL plevel);
 void   SetKernel (PLEVEL plevel, char **args);
 void   SetModelVars (PLEVEL plevel);
-void   SetMCVars (PLEVEL plevel, char **args);
+int    SetMCVars (PLEVEL plevel, char **args);
 int    SumAllExpts (PLEVEL plevel, char **args);
 void   SetPointers (PLEVEL plevel, char **args);
-BOOL   Test (BOOL bExptIsDep, long iter, long initIter, double dLnPrior,
-             double dLnPriorNew, double dLnLike, double dLnLikeNew,
-             double dLnData, double dLnDataNew);
-void TraverseLevels (PLEVEL plevel,
-                     void (*routinePtr)(PLEVEL plevel, char **args), ...);
-int  TraverseLevels1 (PLEVEL plevel,
-                      int (*routinePtr)(PLEVEL plevel, char **args), ...);
-void WriteHeader (PLEVEL plevel, char **args);
-void WriteMCVars (PLEVEL plevel, PANALYSIS panal, PFILE pOutFile);
+BOOL Test (BOOL bExptIsDep, double dLnPrior, double dLnPriorNew,
+           double dLnLike, double dLnLikeNew,
+           double dLnData, double dLnDataNew);
+void   TraverseLevels (PLEVEL plevel,
+                       void (*routinePtr)(PLEVEL plevel, char **args), ...);
+int    TraverseLevels1 (PLEVEL plevel,
+                        int (*routinePtr)(PLEVEL plevel, char **args), ...);
+void   WriteHeader (PLEVEL plevel, char **args);
+void   WriteMCVars (PLEVEL plevel, PANALYSIS panal, PFILE pOutFile);
 
 #endif /* _MH_H_ */
 
