@@ -29,9 +29,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.modelu.c
-    Revision:  1.23
-        Date:  29 Jul 1997
-     Modtime:  08:08:46
+    Revision:  1.25
+        Date:  14 Nov 1997
+     Modtime:  06:39:04
       Author:  @a
    -- SCCS  ---------
 
@@ -154,21 +154,18 @@ void FixupDependentInputs()
       }
 
     /* Fix exponential exposure times for efficiency */
-
     if (vrgInputs[i].iType == IFN_PEREXP) {
       vrgInputs[i].dTexp = N_TAU_EXPOSE / vrgInputs[i].dDecay;
       if (vrgInputs[i].dTper < vrgInputs[i].dTexp)
         vrgInputs[i].dTexp = vrgInputs[i].dTper;
     } /* if */
 
-    /* Assure that input has an off zone at end of period */
-
+    /* FB 9/9/97: Assure that input does not exceed the period */
     if (vrgInputs[i].dTexp != 0.0
         && vrgInputs[i].dT0 + vrgInputs[i].dTexp >= vrgInputs[i].dTper)
-      vrgInputs[i].dTexp = vrgInputs[i].dTper - vrgInputs[i].dT0 - 1e-6;
+      vrgInputs[i].dTexp = vrgInputs[i].dTper - vrgInputs[i].dT0 /* - 1e-6 */;
 
     /* For doses and spikes allow scaling in Scale{} by use of dMag */
-
     if (vrgInputs[i].iType == IFN_NDOSES || vrgInputs[i].iType == IFN_SPIKES)
       vrgInputs[i].dMag = 1.0; /* Initially no scaling */
 
@@ -290,7 +287,8 @@ BOOL UpdateSpikes (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
   register double *rgT0s = pifn->rgT0s; /* The schedule times */
   register int j = pifn->iDoseCur; /* Index to current spike */
 
-  *pdTnext = DBL_MAX; /* If no spikes, make dTnext inf */
+  *pdTnext = DBL_MAX; /* If no spikes, make dTnext infinite */
+  pifn->bOn = FALSE;  /* FB 9/9/97 */
   if (j < pifn->nDoses) {
     if (*pdTime < rgT0s[j])
       *pdTnext = rgT0s[j]; /* Time of upcoming spike */
@@ -306,37 +304,11 @@ BOOL UpdateSpikes (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
       printf ("\nUpdateSpikes: Discontinuity was passed over\n");
   } /* if */
 
-  return (pifn->bOn); /* Flag for post-update of spikes */
+  return (pifn->bOn);
 
 } /* UpdateSpikes */
 
 
-/* -----------------------------------------------------------------------------
-   PostUpdateSpikes
-
-   Reupdate a Spikes type input.
-*/
-void PostUpdateSpikes (PDOUBLE pdTime)
-{
-  static PDOUBLE rgDerivTemp; /* Derivatives are NOT affected */
-  int i;
-
-  if (!(rgDerivTemp))
-    if ( !(rgDerivTemp = InitdVector (GetNModelVars ())))
-      ReportError (NULL, RE_OUTOFMEM | RE_FATAL, "PostUpdateSpikes", NULL);
-
-  CalcDeriv (GetModelVector(), rgDerivTemp, pdTime);
-  for (i = 0; i < vnInputs; i++) /* Turn spikes off */
-    if (vrgInputs[i].iType == IFN_SPIKES
-        && vrgInputs[i].bOn) {
-      vrgInputs[i].bOn = 0;
-      vrgInputs[i].dVal = 0.0;
-      vrgInputs[i].iDoseCur++; /* Advance to next spike */
-    } /* if */
-
-} /* PostUpdateSpikes */
-
-
 /* -----------------------------------------------------------------------------
    UpdateDefaultInput
 
@@ -371,27 +343,25 @@ void UpdateDefaultInput (PIFN pifn, PDOUBLE pdTnext, PDOUBLE pdTime)
 /* -----------------------------------------------------------------------------
    UpdateInputs
 
-   Updates the input according to the current time.
-
-   This routine does not calculate the inputs.  CalcInputs() does
-   that.  This routine updates information about whether inputs are on
-   or off, increments the start of periods for periodic inputs, and
-   returns in *pdNextTransTime the time at which the next input will
-   change states.
-
    The controlling routine to integrator must know when inputs change
    states abruptly to correct for integration across transition
-   boundaries.  This call helps to optimize the calculation of inputs
-   which is done inside the model.
+   boundaries.
 
-   UpdateInputs() assumes that dTime > 0.0, always, and further,
-   that time proceeds positively.
+   Given the current time it returns (in *pdNextTransTime) the time of the 
+   next closest input transition (which is a possible discontinuity).
+
+   It also updates information about whether inputs are on
+   or off, increments the start of periods for periodic inputs.
+
+   It helps optimize the calculation of inputs which is done inside 
+   the model. It does not calculate the inputs.  CalcInputs() does that.  
+
+   UpdateInputs() assumes that time proceeds positively.
 */
 void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
 {
   double dT; /* New time to try */
   int i;
-  BOOL bSpikeOn = FALSE; /* A spike occurs at this time */
 
   if (vbModelReinitd) {         /* If model has been re-initialized */
     FixupDependentInputs ();    /* Fixup input fields dependent on vars */
@@ -411,11 +381,15 @@ void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
         break;
 
       case IFN_SPIKES:
-        /* FB 11/6/97: advance to next spike if we are not at initial time */
-        if (!vbModelReinitd) 
+        /* FB 9/9/97: make it advance only if this input spike is on.
+           At the start it is off. The on/off switch is only used here,
+           CalcInputs does not use it but checks that the current time is 
+           equal to the spike time */
+        if (vrgInputs[i].bOn)
           vrgInputs[i].iDoseCur++;
 
-        bSpikeOn = UpdateSpikes (&vrgInputs[i], &dT, pdTime) || bSpikeOn;
+        UpdateSpikes (&vrgInputs[i], &dT, pdTime);
+
         break;
 
       default:
@@ -434,7 +408,6 @@ void UpdateInputs (PDOUBLE pdTime, PDOUBLE pdNextTransTime)
 
   } /* for */
 
-  /* if (bSpikeOn) PostUpdateSpikes (pdTime); FB 11/06/97: */
   if (vbModelReinitd) { /* If model has just been initialized */
     vbModelReinitd = FALSE; /* don't do it again */
   }    
@@ -758,39 +731,42 @@ void CalcInputs (PDOUBLE pdTime)
   int i;
 
   for (i = 0; i < vnInputs; i++) {
-    if (vrgInputs[i].iType != IFN_CONSTANT /* Constants do not change */
-    && vrgInputs[i].bOn) { /* Exposure must be on */
+
+    /* Constants do not change, exposure must be on, or it's a spike 
+       (always on) */
+    if ((vrgInputs[i].iType != IFN_CONSTANT) && 
+       (vrgInputs[i].bOn || (vrgInputs[i].iType != IFN_SPIKES))) {  
 
       switch (vrgInputs[i].iType) {
-    case IFN_CONSTANT:
-      break; /* dVal does not change */
+        case IFN_CONSTANT:
+          break; /* dVal does not change */
 
-    case IFN_NDOSES:
-      if (vrgInputs[i].iDoseCur < vrgInputs[i].nDoses)
-        vrgInputs[i].dVal = vrgInputs[i].rgMags[vrgInputs[i].iDoseCur] *
-                            vrgInputs[i].dMag; /* Scaled */
-      break;
+        case IFN_NDOSES:
+          if (vrgInputs[i].iDoseCur < vrgInputs[i].nDoses)
+            vrgInputs[i].dVal = vrgInputs[i].rgMags[vrgInputs[i].iDoseCur] *
+                                vrgInputs[i].dMag; /* Scaled */
+          break;
 
-    case IFN_PERDOSE:
-    case IFN_PERRATE: /* Does not change step-wise */
-      break;
+        case IFN_PERDOSE:
+        case IFN_PERRATE: /* Does not change step-wise */
+          break;
 
-    case IFN_PEREXP: /* Calc current exp() */
-      vrgInputs[i].dVal = vrgInputs[i].dMag * 
-                          exp ((vrgInputs[i].dTStartPeriod - *pdTime) * 
-                               vrgInputs[i].dDecay);
-      break;
+        case IFN_PEREXP: /* Calc current exp() */
+          vrgInputs[i].dVal = vrgInputs[i].dMag * 
+                              exp ((vrgInputs[i].dTStartPeriod - *pdTime) * 
+                                   vrgInputs[i].dDecay);
+          break;
 
-    case IFN_SPIKES:
-      if ((*pdTime == vrgInputs[i].rgT0s[vrgInputs[i].iDoseCur]) &&
-          (vrgInputs[i].iDoseCur < vrgInputs[i].nDoses))
-        vrgInputs[i].dVal = vrgInputs[i].rgMags[vrgInputs[i].iDoseCur] *
-                            vrgInputs[i].dMag; /* Scaled */
+        case IFN_SPIKES:
+          if ((*pdTime == vrgInputs[i].rgT0s[vrgInputs[i].iDoseCur]) &&
+              (vrgInputs[i].iDoseCur < vrgInputs[i].nDoses))
+          vrgInputs[i].dVal = vrgInputs[i].rgMags[vrgInputs[i].iDoseCur] *
+                              vrgInputs[i].dMag; /* Scaled */
 
-      else
-        vrgInputs[i].dVal = 0;
+          else
+            vrgInputs[i].dVal = 0;
 
-      break;
+          break;
 
       } /* switch */
     } /* if */
