@@ -29,9 +29,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.modiface.h
-    Revision:  1.9
-        Date:  02 Jul 1997
-     Modtime:  08:37:12
+    Revision:  1.10
+        Date:  14 Nov 1997
+     Modtime:  06:39:18
       Author:  @a
    -- SCCS  ---------
 
