@@ -30,9 +30,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.random.c
-    Revision:  1.27
-        Date:  18 Jul 1997
-     Modtime:  09:22:47
+    Revision:  1.29
+        Date:  14 Nov 1997
+     Modtime:  06:39:06
       Author:  @a
    -- SCCS  ---------
 
@@ -84,9 +84,6 @@
    If the random number generator is not initialized before any of its
    routines are called, it initializes itself with a default seed.
 
-   Modified on 18/3/97 by FB: write "inverse scale" in error message of 
-     ExpRandom 
-
 */
 
 #include <float.h>
@@ -193,8 +190,9 @@ void InitRandom (double dSeed, int bWarmUp)
    An alternative random number generator, so you don't have to use
    the (probably not so good) system supplied standard C version.
 
-   Randoms() returns random numbers between 0 and 1.  The generator can
-   be initialized with InitRandom().
+   Randoms() returns random numbers between 0 and 1. The minimum
+   returned value is 1/m and the maximum 1 - 1/m. The generator can
+   be initialized with InitRandom(). 
 
    This generator should be correct on any system for which the
    representattion of reals uses at least a 32-bit mantissa, including
@@ -208,8 +206,8 @@ double Randoms (void)
 {
 #define a  16807.0
 #define m  2147483647.0
-#define q  127773.0   /* m Div a*/
-#define r  2836.0     /* m Mod a*/
+#define q  127773.0   /* m Div a */
+#define r  2836.0     /* m Mod a */
 
   double hi, test;
 
@@ -315,14 +313,14 @@ double LogNormalRandom (double dMean, double dStdDev)
     char str[10];
     sprintf(str, "%5.2e", dMean);
     ReportRunTimeError(NULL, RE_BADLOGNORMALMEAN | RE_FATAL,
-                       "unknown", str, "LogNormalRandom");
+                       "", str, "LogNormalRandom");
   }
   else 
     if (dStdDev < 1) {
       char str[10];
       sprintf(str, "%5.2e", dStdDev);
       ReportRunTimeError(NULL, RE_BADLOGNORMALSD | RE_FATAL,
-                         "unknown", str, "LogNormalRandom");
+                         "", str, "LogNormalRandom");
     }
 
   return exp (NormalRandom (log (dMean), log (dStdDev)));
@@ -388,7 +386,7 @@ double TruncLogNormalRandom (double dMean, double dStdDev, double a, double b)
 } /* TruncLogNormalRandom */
 
 
-/* -----------------------------------------------------------------------------
+/* ----------------------------------------------------------------------------
    ExpRandom
 
    returns an exponential variate with inverse scale beta
@@ -441,10 +439,13 @@ double GammaRandom (double alpha)
 
     do {
       u1 = b * Randoms();
-
       if (u1 <= 1.0) {
         x = pow(u1, 1./alpha);
-        if (x <= -log(Randoms())) return(x);
+        /* problem: if alpha is too small, x will be about zero and that's
+           bad, particularly for inverse-gamma variates. 
+           Fixed by blocking zeros - FB 5/11/1997 */
+        if ((x > DBL_MIN) && (x <= -log(Randoms()))) 
+          return(x);
       }
       else {
         x = -log((b - u1) / alpha);
