@@ -26,25 +26,21 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.sim.c
-    Revision:  1.46
-        Date:  30 Jul 1997
-     Modtime:  02:13:26
+    Revision:  1.48
+        Date:  14 Nov 1997
+     Modtime:  06:39:06
       Author:  @a
    -- SCCS  ---------
 
    Entry point and main simulation routines for 'sim' program.
 
-   Zeng added lsodes_() in function DoOneExperiment ()
-
-   Modified 18/03/97 by FB in DoMonteCarlo corrected integration failure warning.
-
-   Modified by FB 21/05/1997: disabled simhelp function calls and command-line
-   options; removed dead CheckDependencies code. Removed command-lines options.
-
-   Modified by FB 15/07/97: re-enabled command-line options (but they are 
-   undeveloped) for GNU conformity.
 */
 
+#ifdef _MACOSLEVEL2_
+#include <PPC Macheaders>
+#include "mac.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -58,7 +54,6 @@
 #include "optdesign.h"
 #include "lexerr.h"
 #include "lsodes.h"
-#include "mac.h"
 #include "sim.h"
 /* #include "simhelp.h" */
 #include "simi.h"
@@ -68,7 +63,7 @@
 #include "strutil.h"
 
 
-static char vszVersion[] = "v4.1"; /* Version of program */
+static char vszVersion[] = "v4.2.0"; /* Version of program */
 
 
 /* -----------------------------------------------------------------------------
@@ -226,6 +221,18 @@ int DoOneExperiment (PEXPERIMENT pexp)
 
   UpdateInputs (&pexp->dT0, &dTtrans); /* Resolve dependent inputs */
 
+  if (pexp->dT0 > dTtrans) {
+    printf ("\nError: starting time is greater than first discontinuity,"
+            "       check your inputs - Exiting.\n");
+    exit (0);
+  }
+
+  if (pexp->dT0 > dTout) {
+    printf ("\nError: starting time is greater than first output time,"
+            "       check your outputs - Exiting.\n");
+    exit (0);
+  }
+
   pexp->dTime = pexp->dT0;
 
   /* set lsodes return flag to 1 for first call */
@@ -234,6 +241,11 @@ int DoOneExperiment (PEXPERIMENT pexp)
   /* Iterate to final time */
   while (pexp->dTime < pexp->dTfinal) {
 
+#ifdef _MACOSLEVEL2_
+    /* this allows background processing on the Macintosh */
+    HandleEvent();
+#endif
+
     /* If dynamics equations are defined */
     if (pmod->nStates > 0) {
 
@@ -637,7 +649,7 @@ void PromptFilenames (PSTR *pszFileIn, PSTR *pszFileOut)
   *pszFileOut = (PSTR) calloc (1, 80);
 
   printf ("Input filename? ");
-#ifdef _MACOS_
+#ifdef _MACOSLEVEL1_
   scanf ("%[^:\f\r\v\n]", *pszFileIn);
   getchar();
 #else
@@ -650,7 +662,7 @@ void PromptFilenames (PSTR *pszFileIn, PSTR *pszFileOut)
 
   if ((*pszFileIn)[0]) { /* Input file specified */
     printf ("Output filename? ");
-#ifdef _MACOS_
+#ifdef _MACOSLEVEL1_
     scanf ("%[^:\f\r\v\n]", *pszFileOut);
 #else
     gets (*pszFileOut);
@@ -829,11 +841,9 @@ int main (int nArg, char *const *rgszArg)
   INPUTBUF ibIn;
   PANALYSIS panal = (PANALYSIS) malloc (sizeof(ANALYSIS));
 
-#ifdef _MACOS_
-
-/*InitMacintosh ();
-  while (1) HandleEvent(); */
-
+#ifdef _MACOSLEVEL2_
+  InitMacintosh ();
+  while (1) HandleEvent();
 #endif
 
   AnnounceProgram ();
@@ -861,7 +871,7 @@ int main (int nArg, char *const *rgszArg)
   } /* if */
   FreeLevels(panal);
 
-#ifdef _MACOS_
+#ifdef _MACOSLEVEL1_
   printf ("Done.(Hit Return)\n\n");
 #endif
 
