@@ -28,9 +28,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.sim.h
-    Revision:  1.33
-        Date:  02 Jul 1997
-     Modtime:  08:37:16
+    Revision:  1.35
+        Date:  14 Nov 1997
+     Modtime:  06:39:20
       Author:  @a
    -- SCCS  ---------
 
@@ -59,7 +59,7 @@
 /* These are potential array size problems.
    Other maximum sizes are MAX_EQN and MAX_LEX in lex.h */
 
-#define MAX_LEVELS            10   /* for now; actual is 255 */
+#define MAX_LEVELS            10   /* for now; actual possible is 255 */
 #define MAX_INSTANCES         200  /* for now; allowable instances of `Level'
                                       at any depth */
 #define MAX_EXPERIMENTS       200
@@ -67,7 +67,7 @@
 #define LSODES_RWORKSIZE      1200 /* Idem */
 #define ARGS_MAX              8    /* Maximum number of args to lex */
                                    /* Used in allocating argument strings */
-#define MAX_PRINT_VARS        5    /* Arbitrary */
+#define MAX_PRINT_VARS        10   /* Arbitrary */
 
 /* Keyword Map constants */
 
