@@ -29,19 +29,14 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.simi.c
-    Revision:  1.46
-        Date:  29 Jul 1997
-     Modtime:  08:08:47
+    Revision:  1.48
+        Date:  14 Nov 1997
+     Modtime:  06:39:07
       Author:  @a
    -- SCCS  ---------
 
    Contains input routines for simulation.
 
-   Modified on 18/3/97 by FB: improved comment for exponential density in 
-     GetMCVarySpec.
-
-   Modified on 21/3/97 by FB: made undefined identifier a fatal error, 
-     and collapsed two memory allocation checking in one, in GetPrint.
 */
 
 #include <assert.h>
@@ -478,25 +473,25 @@ BOOL GetListOfTimes (PINPUTBUF pibIn, int nRecs, PPRINTREC *ppr, PSTR szLex)
   } while ((iNLI = NextListItem (pibIn, szLex, LX_NUMBER, 1, CH_RPAREN)) > 0);
 
   if (!iNLI) /* List terminator */
-    bErr = EGetPunct (pibIn, szLex, CH_RPAREN)
-           || !ListLength(plistTimes);
+    bErr = EGetPunct (pibIn, szLex, CH_RPAREN) || !ListLength(plistTimes);
   else {
     bErr = TRUE;
     ReportError (pibIn, RE_LEXEXPECTED, "number", szLex);
-  } /* else */
+  }
 
   if (!bErr) 
     for(i = 0; i < nRecs; ++i)
       DListToArray (plistTimes, &ppr[i]->cTimes, &ppr[i]->pdTimes);
 
-  for (i = 1; i < ppr[0]->cTimes && !bErr; i++)    /* Verify Times */
+  FreeList (&plistTimes, NULL, TRUE); /* Free list and cells */
+
+  for (i = 1; i < ppr[0]->cTimes && !bErr; i++) /* Verify Times */
     if ((bErr = (*(ppr[0]->pdTimes+i) <= *(ppr[0]->pdTimes+i-1)))) {
-      ReportError (pibIn, RE_SPECERR, "Times out of order", NULL);
       for(j = 0; j < nRecs; ++j)
         free (ppr[j]->pdTimes);
+      ReportError (pibIn, RE_SPECERR | RE_FATAL, "Times out of order", NULL);
     } /* if */
 
-  FreeList (&plistTimes, NULL, TRUE); /* Free list and cells */
   return (bErr);
 
 } /* GetListOfTimes */
@@ -600,7 +595,7 @@ BOOL GetPrint (PINPUTBUF pibIn, PSTR szLex, POUTSPEC pos)
   else {
     if (!bGavePrintUsage) {
       printf ("Syntax: %s (identifier, Time1, Time2, ...)\n\n",
-           GetKeyword(KM_PRINT));
+              GetKeyword(KM_PRINT));
       bGavePrintUsage = TRUE;
     }
   } /* else */
@@ -1683,7 +1678,12 @@ void NewExperiment (PINPUTBUF pibIn)
 
   /* Allocate new experiment and assign list and current pointers */
 
-  if(panal->iCurrentDepth == 0) {
+  if (panal->iCurrentDepth < 0) { /* something is real wrong - FB 03/08/97 */
+     ReportError (pibIn, RE_LEXEXPECTED | RE_FATAL, "Level statement", 
+                  "Experiment");
+  }
+
+  if (panal->iCurrentDepth == 0) {
     panal->expGlobal.iExp++;    /* Increment number of experiment */
     panal->pexpCurrent = panal->rgpExps[panal->expGlobal.iExp - 1] =
                          (PEXPERIMENT) malloc (sizeof(EXPERIMENT));
@@ -1694,31 +1694,37 @@ void NewExperiment (PINPUTBUF pibIn)
   }
   else {
     plevel = panal->pLevels[panal->iInstances-1];
-    for(n = 0; n < panal->iCurrentDepth-1; ++n) {
+    for (n = 0; n < panal->iCurrentDepth-1; ++n) {
       plevel = plevel->pLevels[plevel->iInstances-1];
     }
-    if(plevel->iInstances == MAX_INSTANCES - 1)
+    if (plevel->iInstances == MAX_INSTANCES - 1)
       ReportError(pibIn, RE_TOOMANYINST | RE_FATAL, "NewExperiment", NULL);
     n = panal->pCurrentLevel[panal->iCurrentDepth-1]->iInstances++;
-    if((plevel = plevel->pLevels[n] = (PLEVEL)malloc(sizeof(LEVEL))) == NULL)
+    if ((plevel = plevel->pLevels[n] = (PLEVEL)malloc(sizeof(LEVEL))) == NULL)
       ReportError(pibIn, RE_OUTOFMEM | RE_FATAL, "NewExperiment", NULL);
     plevel->iInstances = 0;
     plevel->iSequence = n + 1;
     plevel->iDepth = panal->iCurrentDepth;
     panal->pCurrentLevel[panal->iCurrentDepth++] = plevel;
-    if(panal->iDepth < panal->iCurrentDepth)
+    if (panal->iDepth < panal->iCurrentDepth)
       panal->iDepth = panal->iCurrentDepth;
+
+    plevel->nMCVars = plevel->nFixedVars = 0; /* FB 13 aug 97 */
     plevel->plistVars = InitList();
     plevel->plistMCVars = InitList();
-    if((plevel->pexpt = (PEXPERIMENT)malloc(sizeof(EXPERIMENT))) == NULL)
+
+    if ((plevel->pexpt = (PEXPERIMENT)malloc(sizeof(EXPERIMENT))) == NULL)
       ReportError(pibIn, RE_OUTOFMEM | RE_FATAL, "NewExperiment", NULL);
+
     panal->pexpCurrent = plevel->pexpt;
     panal->pexpCurrent->iExp = panal->expGlobal.iExp = ++panal->iExpts;
     panal->wContext = CN_EXPERIMENT;
+
     printf ("Experiment %d - depth %d, instance %d\n", panal->iExpts,
             panal->iCurrentDepth,
             panal->pCurrentLevel[panal->iCurrentDepth-2]->iInstances);
   }
+
   memcpy (panal->pexpCurrent, &panal->expGlobal, sizeof(EXPERIMENT));
   panal->wContext = CN_EXPERIMENT;
   panal->pexpCurrent->plistParmMods = InitList();    /* Local mods */
