@@ -28,9 +28,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.simi.h
-    Revision:  1.9
-        Date:  02 Jul 1997
-     Modtime:  08:37:19
+    Revision:  1.10
+        Date:  14 Nov 1997
+     Modtime:  06:39:20
       Author:  @a
    -- SCCS  ---------
 
