@@ -28,9 +28,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.yourcode.c
-    Revision:  1.17
-        Date:  28 Jul 1997
-     Modtime:  04:23:12
+    Revision:  1.19
+        Date:  14 Nov 1997
+     Modtime:  06:39:11
       Author:  @a
    -- SCCS  ---------
 
@@ -38,13 +38,17 @@
    
 */
 
+#ifdef _MACOSLEVEL2_
+#include <PPC Macheaders>
+#include "mac.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <math.h>
 
 #include "lexerr.h"
-#include "mac.h"   /* _MACOS_ */
 #include "simmonte.h"
 #include "yourcode.h"
 
@@ -72,11 +76,9 @@ void DoStep_by_Step (/* double t, long *neq, double *y */)
   
   */
 
-#ifdef _MACOS_
-
-  /* this allows background processing on the Macintosh
-  HandleEvent(); */
-
+#ifdef _MACOSLEVEL2_
+  /* this allows background processing on the Macintosh */
+  /* HandleEvent(); */
 #endif
 
 } /* DoStep_by_Step */
