@@ -28,9 +28,9 @@
 
    -- Revisions -----
      Logfile:  SCCS/s.yourcode.h
-    Revision:  1.6
-        Date:  18 Jul 1997
-     Modtime:  09:28:40
+    Revision:  1.7
+        Date:  14 Nov 1997
+     Modtime:  06:39:23
       Author:  @a
    -- SCCS  ---------
 
